@@ -17,10 +17,11 @@
  *   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
-/* Define if no regulation domain restrictions. */
+/* Define if no regulation domain restrictions */
 #undef NO_REG_DOMAIN
 
-/* Define if OS Linux. */
+/* Define if OS Linux */
 #undef OS_LINUX
 
-
+/* Define if OS OpenBSD */
+#undef OS_BSD
