@@ -61,20 +61,20 @@ void EthStat()
     int j, child_pipe[2];
     varbind varbinds[2];
 
-	if (ME102 == ap_type) {
-		EthRx[5] = 0xe0;
-		EthRx[6] = 0x3e;
-		EthTx[5] = 0xe0;
-		EthTx[6] = 0x3e;
-	}
+    if (ap_type == ATMEL12350) {
+	EthRx[5] = 0xE0;
+	EthRx[6] = 0x3E;
+	EthTx[5] = 0xE0;
+	EthTx[6] = 0x3E;
+    }
     
     if (pipe(child_pipe) == -1) {
-	print_help(PIPE);
+	print_helperr(PIPE);
 	goto exit;
     }
 
     if (fcntl(child_pipe[0], F_SETFL, O_NONBLOCK) == -1) {
-	print_help(FCNTL);
+	print_helperr(FCNTL);
 	goto exit;
     }
     print_title(_("Ethernet Statistics"));
@@ -83,7 +83,7 @@ void EthStat()
 
     switch (pid = fork()) {
     case -1:
-	print_help(FORK);
+	print_helperr(FORK);
 	goto exit;
     case 0:
 	close(child_pipe[1]);
@@ -101,7 +101,7 @@ void EthStat()
 	varbinds[1].type = NULL_VALUE;
 
 	if (snmp(varbinds, 2, GET) <= 0) {
-	    print_help(ERR_RET_PRESS);
+	    print_helperr(ERR_RET_PRESS);
 	    goto exit_child;
 	}
 
@@ -113,7 +113,7 @@ void EthStat()
 						       len_val);
 	    memcpy(EthRxStat, varbinds[0].value, varbinds[0].len_val);
 	} else {
-	    print_help(_("EthRxStat packet error. Press Q to continue."));
+	    print_helperr(_("EthRxStat packet error. Press Q to continue."));
 	    goto exit_child;
 	}
 
@@ -125,11 +125,11 @@ void EthStat()
 						       len_val);
 	    memcpy(EthTxStat, varbinds[1].value, varbinds[1].len_val);
 	} else {
-	    print_help(_("EthTxStat packet error. Press Q to continue."));
+	    print_helperr(_("EthTxStat packet error. Press Q to continue."));
 	    goto exit_child;
 	}
 	mvwaddstr(main_sub, 1, 2, _("Received:"));
-	mvwaddstr(main_sub, 1, 30, _("Transmited:"));
+	mvwaddstr(main_sub, 1, 30, _("Transmitted:"));
 	sprintf(message, "TotalBytes       %10u TotalBytes         %10u",
 		swap4(EthRxStat->TotalBytesRx),
 		swap4(EthTxStat->TotalBytesTx));
@@ -242,18 +242,18 @@ void WirelessStat()
 
     varbind varbinds[1];
 
-	if (ME102 == ap_type) {
-		Wireless[5] = 0xe0;
-		Wireless[6] =0x3e;
-	}
-    
+    if (ap_type == ATMEL12350) {
+	Wireless[5] = 0xE0;
+	Wireless[6] = 0x3E;
+    }
+
     if (pipe(child_pipe) == -1) {
-	print_help(PIPE);
+	print_helperr(PIPE);
 	goto exit;
     }
 
     if (fcntl(child_pipe[0], F_SETFL, O_NONBLOCK) == -1) {
-	print_help(FCNTL);
+	print_helperr(FCNTL);
 	goto exit;
     }
     print_title(_("Wireless Statistics"));
@@ -262,7 +262,7 @@ void WirelessStat()
 
     switch (pid = fork()) {
     case -1:
-	print_help(FORK);
+	print_helperr(FORK);
 	goto exit_child;
     case 0:
 	close(child_pipe[1]);
@@ -275,7 +275,7 @@ void WirelessStat()
 	varbinds[0].type = NULL_VALUE;
 
 	if (snmp(varbinds, 1, GET) <= 0) {
-	    print_help(ERR_RET_PRESS);
+	    print_helperr(ERR_RET_PRESS);
 	    goto exit_child;
 	}
 
@@ -286,7 +286,7 @@ void WirelessStat()
 		(struct WirelessStatistics *) malloc(varbinds[0].len_val);
 	    memcpy(WirelessStat, varbinds[0].value, varbinds[0].len_val);
 	} else {
-	    print_help
+	    print_helperr
 		(_("WirelessStat packet error. Press Q to continue."));
 	    goto exit;
 	}
@@ -424,12 +424,12 @@ void nwn_wireless_stat()
     curs_set(0);
 
     if (pipe(child_pipe) == -1) {
-	print_help(_("pipe error. Press any key"));
+	print_helperr(_("pipe error. Press any key"));
 	goto exit;
     }
 
     if (fcntl(child_pipe[0], F_SETFL, O_NONBLOCK) == -1) {
-	print_help(_("fcntl error. Press any key"));
+	print_helperr(_("fcntl error. Press any key"));
 	goto exit;
     }
     print_title(_("Wireless Statistics"));
@@ -438,7 +438,7 @@ void nwn_wireless_stat()
 
     switch (pid = fork()) {
     case -1:
-	print_help(_("fork error. Press any key"));
+	print_helperr(_("fork error. Press any key"));
 	goto exit_child;
     case 0:
 	close(child_pipe[1]);
@@ -484,7 +484,7 @@ void nwn_wireless_stat()
 	    varbinds[i].type = 0x05;
 	}
 	if (snmp(varbinds, 16, GET) <= 0) {
-	    print_help(ERR_RET_PRESS);
+	    print_helperr(ERR_RET_PRESS);
 	    goto exit_child;
 	}
 
