@@ -20,23 +20,47 @@
 #include <ncurses.h>
 #include <string.h>
 #include <sys/types.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
 #include <stdlib.h>
-#include "ap-utils.h"
 #include <menu.h>
 #include <unistd.h>
+#include "ap-utils.h"
 
-#define HIDE_ESSID _("[S] Insert ESSID in broadcast packets: %s")
-#define RETRAIN _("[U] AutoRateFallBack: %s")
-#define PREAMBULE _("[P] Preambule type: %s")
-#define RATES_RECORD "[%d]  %4.1fM   %s"
+#define ESSID _("[E] ESSID: ")
+#define AP_NAME _("[N] AP name: ")
+
+#define CONTACT _("[E] AP contact: ")
+#define LOCATION _("[L] AP location: ")
+
+#define RTS_TR _("[R] RTS threshold: ")
+#define FRG_TR _("[F] Fragmentation threshold: ")
+#define PREAMBULE _("[P] Preambule type: ")
+#define AUTH _("[A] Auth type: ")
+#define   OSYS _("Open system")
+#define   SH_KEY _("Shared key")
+#define   BOTH_TYPE _("Both types")
+#define RETRAIN _("[U] Auto rate fallback: ")
+#define HIDE_ESSID _("[S] Insert ESSID in broadcast packets: ")
+#define RATES _("Basic and Supported rates:")
+#define RATES_HEAD _("Key   Rate  Status")
+#define   RATES_RECORD "[%d]  %4.1fM   "
+#define INT_ROAMING _("[I] International roaming: ")
+#define BEACON_PER _("[B] Beacon period (msec): ")
+#define DTIM_I _("[D] DTIM sending interval (beacons): ")
+#define WLAN_HELP _("[key] - set option; W - write conf; Q - quit to menu")
 
 extern short ap_type;
 
-void wireless()
+void atmel_wireless()
 {
+    char sysDeviceInfo[] =
+	{ 0x2B, 0x06, 0x01, 0x04, 0x01, 0x83, 0x1A, 0x01, 0x01, 0x01, 0x05,
+	0x00
+    };
+    char operEthernetAddress[] =
+        { 0x2B, 0x06, 0x01, 0x04, 0x01, 0x83, 0x1A, 0x01, 0x01, 0x02, 0x03,
+        0x00
+    };
+
     char operChannelID[] =
 	{ 0x2B, 0x06, 0x01, 0x04, 0x01, 0x83, 0x1A, 0x01, 0x02, 0x01, 0x01,
 	0x00
@@ -81,137 +105,202 @@ void wireless()
 	{ 0x2B, 0x06, 0x01, 0x04, 0x01, 0x83, 0x1A, 0x01, 0x02, 0x01, 0x0B,
 	0x00
     };
-    char sysDeviceInfo[] =
-	{ 0x2B, 0x06, 0x01, 0x04, 0x01, 0x83, 0x1A, 0x01, 0x01,
-	0x01, 0x05, 0x00
+
+    /* This one is ATMEL12350 MIB specific. */
+    char operInterRoaming[] =
+	{ 0x2B, 0x06, 0x01, 0x04, 0x01, 0xE0, 0x3E, 0x01, 0x02, 0x01, 0x0D,
+	0x00
+    };
+    /* These two are ATMEL12350 TELLUS MIB specific. */
+    char operBeaconPeriod[] =
+	{ 0x2B, 0x06, 0x01, 0x04, 0x01, 0xE0, 0x3E, 0x01, 0x02, 0x01, 0x0E,
+	0x00
+    };
+    char operDTIM[] =
+	{ 0x2B, 0x06, 0x01, 0x04, 0x01, 0xE0, 0x3E, 0x01, 0x02, 0x01, 0x0F,
+	0x00
     };
 
-    struct HARD_CONFIGURATION_STUCT *HardConf;
-    unsigned short int i, RTSThreshold, FragmentationThreshold,
-	m_basic_rates = 0, first_channel=1;
-    char *AuthenticationTypes[3] =
-	{ OSYS, SH_KEY, BOTH_TYPE }, message[1024];
     extern WINDOW *main_sub;
+    varbind varbinds[15];
+    unsigned short int RTSThreshold, FragmentationThreshold,
+	InterRoaming, m_basic_rates = 0, first_channel = 1;
+    unsigned int BeaconPeriod = 0;
+    unsigned char dtim;
+    char *AuthenticationTypes[3] = { OSYS, SH_KEY, BOTH_TYPE },
+	message[1024],
+	oui_tellus[3] = { 0x00, 0x04, 0xDB },
+	IS_ATMEL12350_TELLUS = 0; /* ATMEL12350 Tellus uses different MIB */
     char domain[32], basic_rates[4], AutoRateFallBack, SSIDBroadcasting,
-	ap_name[32], ch_num = 0;
-    char m_channel = 0, m_essid = 0, m_broadcast = 0, m_rts =
-	0, m_fragment = 0, m_auth = 0, m_ap_name = 0, m_preambule =
-	0, m_auto_rate = 0, PreambleType, AuthenticationType, channel;
-    varbind varbinds[11];
+	ap_name[32], ch_num = 0, PreambleType, AuthenticationType, channel;
+    char m_channel = 0, m_essid = 0, m_broadcast = 0, m_rts = 0,
+	m_fragment = 0, m_auth = 0, m_ap_name = 0, m_preambule = 0,
+	m_auto_rate = 0, m_inter_roaming = 0, m_beacon = 0, m_dtim = 0;
     char *item_name[14], *rates[3] = { ON, OFF, BASIC }, *preambules[2] = {
     _("Short"), _("Long")};
     short __rates[4] = { 2, 4, 11, 22 };
-    int c=0;
+    int i, c = 0;
+    unsigned int ap_regdomain;
 
-	if (ME102 == ap_type) {
-	    	operChannelID[5] = 0xE0;
-    		operESSIDLength[5] = 0xE0;
-    		operESSID[5] = 0xE0;
-    		operRTSThreshold[5] = 0xE0;
-    		operFragmentationThreshold[5] = 0xE0;
-    		operPreambleType[5] = 0xE0;
-    		operAuthenticationType[5] = 0xE0;
-    		operBasicRates[5] = 0xE0;
-    		operAutoRateFallBack[5] = 0xE0;
-    		operAccessPointName[5] = 0xE0;
-    		operSSIDBroadcasting[5] = 0xE0;
-    		sysDeviceInfo[5] = 0xE0;
-                operChannelID[6] = 0x3e;
-                operESSIDLength[6] = 0x3e;
-                operESSID[6] = 0x3e;
-                operRTSThreshold[6] = 0x3e;
-                operFragmentationThreshold[6] = 0x3e;
-                operPreambleType[6] = 0x3e;
-                operAuthenticationType[6] = 0x3e;
-                operBasicRates[6] = 0x3e;
-                operAutoRateFallBack[6] = 0x3e;
-                operAccessPointName[6] = 0x3e;
-                operSSIDBroadcasting[6] = 0x3e;
-                sysDeviceInfo[6] = 0x3e;
-	}
+    if (ap_type == ATMEL12350) {
+	sysDeviceInfo[5] = 0xE0;
+	sysDeviceInfo[6] = 0x3E;
+	operEthernetAddress[5] = 0xE0;
+	operEthernetAddress[6] = 0x3E;
+	operChannelID[5] = 0xE0;
+	operChannelID[6] = 0x3E;
+	operESSIDLength[5] = 0xE0;
+	operESSIDLength[6] = 0x3E;
+	operESSID[5] = 0xE0;
+	operESSID[6] = 0x3E;
+	operRTSThreshold[5] = 0xE0;
+	operRTSThreshold[6] = 0x3E;
+	operFragmentationThreshold[5] = 0xE0;
+	operFragmentationThreshold[6] = 0x3E;
+	operPreambleType[5] = 0xE0;
+	operPreambleType[6] = 0x3E;
+	operAuthenticationType[5] = 0xE0;
+	operAuthenticationType[6] = 0x3E;
+	operBasicRates[5] = 0xE0;
+	operBasicRates[6] = 0x3E;
+	operAutoRateFallBack[5] = 0xE0;
+	operAutoRateFallBack[6] = 0x3E;
+	operAccessPointName[5] = 0xE0;
+	operAccessPointName[6] = 0x3E;
+	operSSIDBroadcasting[5] = 0xE0;
+	operSSIDBroadcasting[6] = 0x3E;
+    }
+
+
+    /* Find out whether the device uses enhanced ATMEL12350 TELLUS MIB. */
+    varbinds[0].type = NULL_VALUE;
+    varbinds[0].oid = operEthernetAddress;
+    varbinds[0].len_oid = sizeof(operEthernetAddress);
+    varbinds[0].len_val = 0;
     
-    print_title(_("Wireless Settings"));
     print_help(WAIT_RET);
+    if (snmp(varbinds, 1, GET) <= 0) {
+	print_helperr(ERR_RET);
+	goto exit;
+    }
 
-    for (i = 0; i < 11; i++) {
+    if (ap_type == ATMEL12350 && (memcmp(oui_tellus, varbinds[0].value, 3) ==0))
+	IS_ATMEL12350_TELLUS = 1;
+
+
+    for (i = 1; i < 15; i++) {
 	varbinds[i].type = NULL_VALUE;
 	varbinds[i].len_val = 0;
 	varbinds[i].len_oid = sizeof(operChannelID);
     }
-    varbinds[0].oid = operChannelID;
-    varbinds[1].oid = operESSID;
-    varbinds[2].oid = operSSIDBroadcasting;
-    varbinds[3].oid = operBasicRates;
-    varbinds[4].oid = operAutoRateFallBack;
-    varbinds[5].oid = operRTSThreshold;
-    varbinds[6].oid = operFragmentationThreshold;
-    varbinds[7].oid = operPreambleType;
-    varbinds[8].oid = operAuthenticationType;
-    varbinds[9].oid = operAccessPointName;
-    varbinds[10].oid = sysDeviceInfo;
 
+    i = 0;
 
-    if (snmp(varbinds, 11, GET) <= 0) {
-	print_help(ERR_RET);
+    varbinds[i++].oid = operChannelID;
+    varbinds[i++].oid = operESSIDLength;
+    varbinds[i++].oid = operESSID;
+    varbinds[i++].oid = operSSIDBroadcasting;
+    varbinds[i++].oid = operBasicRates;
+    varbinds[i++].oid = operAutoRateFallBack;
+    varbinds[i++].oid = operRTSThreshold;
+    varbinds[i++].oid = operFragmentationThreshold;
+    varbinds[i++].oid = operPreambleType;
+    varbinds[i++].oid = operAuthenticationType;
+    varbinds[i++].oid = operAccessPointName;
+    varbinds[i++].oid = sysDeviceInfo;
+    if (ap_type == ATMEL12350) {
+	varbinds[i++].oid = operInterRoaming;
+	if (IS_ATMEL12350_TELLUS) {
+	    varbinds[i++].oid = operBeaconPeriod;
+	    varbinds[i++].oid = operDTIM;
+	}
+    }
+
+    if (snmp(varbinds, i, GET) <= 0) {
+	print_helperr(ERR_RET);
 	goto exit;
     }
 
     channel = *(varbinds[0].value);
-    memcpy(domain, varbinds[1].value, 32);
-    memcpy(basic_rates, varbinds[3].value, 4);
-    AutoRateFallBack = *(varbinds[4].value);
-    SSIDBroadcasting = *(varbinds[2].value);
-    memcpy(&RTSThreshold, (varbinds[5].value), 2);
+    c = *(varbinds[1].value);
+    memcpy(domain, varbinds[2].value, 32);
+    SSIDBroadcasting = *(varbinds[3].value);
+    memcpy(basic_rates, varbinds[4].value, 4);
+    AutoRateFallBack = *(varbinds[5].value);
+    memcpy(&RTSThreshold, varbinds[6].value, 2);
     RTSThreshold = ntohs(RTSThreshold);
-    memcpy(&FragmentationThreshold, (varbinds[6].value), 2);
+    memcpy(&FragmentationThreshold, varbinds[7].value, 2);
     FragmentationThreshold = ntohs(FragmentationThreshold);
-    PreambleType = *(varbinds[7].value);
-    AuthenticationType = *(varbinds[8].value);
-    memcpy(ap_name, varbinds[9].value, 32);
+    PreambleType = *(varbinds[8].value);
+    AuthenticationType = *(varbinds[9].value);
+    memcpy(ap_name, varbinds[10].value, 32);
+    if (ap_type == ATMEL12350) {
+	InterRoaming = *(varbinds[12].value);
+	if (IS_ATMEL12350_TELLUS) {
+	    for (i = 0; i < varbinds[13].len_val;
+		BeaconPeriod += varbinds[13].value[i] *
+		(1 << ((varbinds[13].len_val - i - 1) * 8)), i++);
+	    dtim = *(varbinds[14].value);
+	}
+    }
 
-    HardConf =
-	(struct HARD_CONFIGURATION_STUCT *) malloc(varbinds[10].len_val);
-    memcpy(HardConf, varbinds[10].value, varbinds[10].len_val);
-    HardConf->RegulationDomain = swap4(HardConf->RegulationDomain);
-    if ((ch_num = ch_list(HardConf->RegulationDomain, item_name)) == 0) {
-	print_help(ERR_RET);
+    if (ap_type == ATMEL12350)
+	ap_regdomain =
+	    ((struct SysDeviceInfo_ATMEL12350 *) varbinds[11].value)->RegulatoryDomain;
+    else /* ATMEL410 */
+	ap_regdomain =
+	    ((struct SysDeviceInfo_ATMEL410 *) varbinds[11].value)->RegulationDomain;
+
+    ap_regdomain = swap4(ap_regdomain);
+    if ((ch_num = ch_list(ap_regdomain, item_name)) == 0) {
+	print_helperr(ERR_RET);
 	goto exit;
     }
-    if (HardConf->RegulationDomain == 0x31 || HardConf->RegulationDomain == 0x32)
+    if (ap_regdomain == 0x31 || ap_regdomain == 0x32)
 	first_channel=10;
-    free(HardConf);
 
-    sprintf(message, CHANNEL, channel, 2407 + 5 * channel);
+    print_title(_("Wireless Settings"));
+
+    sprintf(message, "%s%02u (%u MHz)", CHANNEL, channel, 2407 + 5 * channel);
     mvwaddstr(main_sub, 0, 0, message);
     mvwaddstr(main_sub, 1, 0, ESSID);
-    for (i = 0; i < 32 && *(domain + i); i++) {
-	mvwaddch(main_sub, 1, i + 24, *(domain + i));
-    }
-    mvwaddstr(main_sub, 2, 0, BSS_NAME);
-    for (i = 0; i < 32 && *(ap_name + i); i++)
-	mvwaddch(main_sub, 2, i + 21, *(ap_name + i));
-    sprintf(message, RTS_TR, RTSThreshold);
+    waddnstr(main_sub, domain, c);
+    mvwaddstr(main_sub, 2, 0, AP_NAME);
+    for (i = 0; i < 32 && ap_name[i]; i++)
+	mvwaddch(main_sub, 2, strlen(AP_NAME) + i, ap_name[i]);
+    sprintf(message, "%s%d", RTS_TR, RTSThreshold);
     mvwaddstr(main_sub, 3, 0, message);
-    sprintf(message, FRG_TR, FragmentationThreshold);
+    sprintf(message, "%s%d", FRG_TR, FragmentationThreshold);
     mvwaddstr(main_sub, 4, 0, message);
-    sprintf(message, PREAMBULE, preambules[PreambleType - 1]);
+    sprintf(message, "%s%s", PREAMBULE, preambules[PreambleType - 1]);
     mvwaddstr(main_sub, 5, 0, message);
-    sprintf(message, AUTH, AuthenticationTypes[AuthenticationType - 1]);
+    sprintf(message, "%s%s", AUTH, AuthenticationTypes[AuthenticationType - 1]);
     mvwaddstr(main_sub, 6, 0, message);
-    sprintf(message, RETRAIN, (AutoRateFallBack == 1) ? ON : OFF);
+    sprintf(message, "%s%s", RETRAIN, (AutoRateFallBack == 1) ? ON : OFF);
     mvwaddstr(main_sub, 7, 0, message);
-    sprintf(message, HIDE_ESSID, (SSIDBroadcasting == 1) ? ON : OFF);
+    sprintf(message, "%s%s", HIDE_ESSID, (SSIDBroadcasting == 1) ? ON : OFF);
     mvwaddstr(main_sub, 8, 0, message);
     mvwaddstr(main_sub, 9, 0, RATES);
     mvwaddstr(main_sub, 10, 2, RATES_HEAD);
     for (i = 0; i < 4; i++) {
-	sprintf(message, RATES_RECORD, i + 1, (float) __rates[i] / 2,
+	sprintf(message, RATES_RECORD "%s", i + 1, (float) __rates[i] / 2,
 		basic(basic_rates[i]));
 	mvwaddstr(main_sub, 11 + i, 2, message);
     }
+    if (ap_type == ATMEL12350) {
+	sprintf(message, "%s%s", INT_ROAMING, (InterRoaming == 1) ? ON : OFF);
+	mvwaddstr(main_sub, 15, 0, message);
+	if (IS_ATMEL12350_TELLUS) {
+	    sprintf(message, "%s%u", BEACON_PER, BeaconPeriod);
+	    mvwaddstr(main_sub, 16, 0, message);
+	    sprintf(message, "%s%u", DTIM_I, dtim);
+	    mvwaddstr(main_sub, 17, 0, message);
+	}
+    }
+
     wrefresh(main_sub);
     noecho();
-    print_help(_("[key] - set option; W - write conf; Q - quit to menu"));
+    print_help(WLAN_HELP);
 
     while (1) {
 	c = getch();
@@ -236,86 +325,104 @@ void wireless()
 	    case 2:
 		basic_rates[i - 1] = __rates[i - 1] + 0x80;
 	    }
-	    sprintf(message, RATES_RECORD, i, (float) __rates[i - 1] / 2,
-		    basic(basic_rates[i - 1]));
 	    clear_main_new(10 + i, 11 + i);
-	    mvwaddstr(main_sub, 10 + i, 2, message);
-	    wrefresh(main_sub);
+	    sprintf(message, RATES_RECORD, i, (float) __rates[i - 1] / 2);
+	    print_menusel(10 + i, 2, message, basic(basic_rates[i - 1]));
 	    m_basic_rates = 1;
 	    break;
 	case 'C':
 	case 'c':
-	    channel = menu_choose(22, 0, item_name, ch_num) + first_channel;
+	    channel = menu_choose(strlen(CHANNEL), 0, item_name, ch_num)
+		+ first_channel;
+	    sprintf(message, "%02u (%u MHz)", channel, 2407 + 5 * channel);
+	    print_menusel(0, 0, CHANNEL, message);
 	    m_channel = 1;
-	    sprintf(message, CHANNEL, channel, 2407 + 5 * channel);
-	    mvwaddstr(main_sub, 0, 0, message);
-	    wrefresh(main_sub);
 	    continue;
 	case 'A':
 	case 'a':
 	    AuthenticationType =
-		menu_choose(15, 6, AuthenticationTypes, 3) + 1;
-	    m_auth = 1;
+		menu_choose(strlen(AUTH), 6, AuthenticationTypes, 3) + 1;
 	    clear_main_new(6, 7);
-	    sprintf(message, AUTH,
+	    print_menusel(6, 0, AUTH,
 		    AuthenticationTypes[AuthenticationType - 1]);
-	    mvwaddstr(main_sub, 6, 0, message);
-	    wrefresh(main_sub);
+	    m_auth = 1;
 	    continue;
 	case 'P':
 	case 'p':
-	    PreambleType = menu_choose(20, 5, preambules, 2) + 1;;
-	    m_preambule = 1;
+	    PreambleType = menu_choose(strlen(PREAMBULE), 5, preambules, 2) + 1;
 	    clear_main_new(5, 6);
-	    sprintf(message, PREAMBULE, preambules[PreambleType - 1]);
-	    mvwaddstr(main_sub, 5, 0, message);
-	    wrefresh(main_sub);
+	    print_menusel(5, 0, PREAMBULE, preambules[PreambleType - 1]);
+	    m_preambule = 1;
 	    continue;
 	case 'U':
 	case 'u':
-	    AutoRateFallBack = on_off(22, 7);
-	    m_auto_rate = 1;
+	    AutoRateFallBack = on_off(strlen(RETRAIN), 7);
 	    clear_main_new(7, 8);
-	    sprintf(message, RETRAIN, (AutoRateFallBack == 1) ? ON : OFF);
-	    mvwaddstr(main_sub, 7, 0, message);
-	    wrefresh(main_sub);
+	    print_menusel(7, 0, RETRAIN, (AutoRateFallBack == 1) ? ON : OFF);
+	    m_auto_rate = 1;
 	    continue;
 	case 'S':
 	case 's':
-	    SSIDBroadcasting = on_off(39, 8);
-	    m_broadcast = 1;
+	    SSIDBroadcasting = on_off(strlen(HIDE_ESSID), 8);
 	    clear_main_new(8, 9);
-	    sprintf(message, HIDE_ESSID,
-		    (SSIDBroadcasting == 1) ? ON : OFF);
-	    mvwaddstr(main_sub, 8, 0, message);
-	    wrefresh(main_sub);
+	    print_menusel(8, 0, HIDE_ESSID, (SSIDBroadcasting == 1) ? ON : OFF);
+	    m_broadcast = 1;
 	    continue;
 	case 'N':
 	case 'n':
-//          make_field(2, 21, 32);
-	    get_value(ap_name, 2, 21, 32);
+//          make_field(2, strlen(AP_NAME), 32);
+	    get_value(ap_name, 2, strlen(AP_NAME), 32, ANY_STRING, 0, 0, NULL);
 	    m_ap_name = 1;
 	    continue;
-	case 'D':
-	case 'd':
-//          make_field(1, 24, 32);
-	    get_value(domain, 1, 24, 32);
+	case 'E':
+	case 'e':
+//          make_field(1, strlen(ESSID), 32);
+	    get_value(domain, 1, strlen(ESSID), 32, ANY_STRING, 0, 0, NULL);
 	    m_essid = 1;
 	    continue;
 	case 'F':
 	case 'f':
-//          make_field(4, 29, 6);
-	    get_value(message, 4, 29, 6);
+//          make_field(4, strlen(FRG_TR), 6);
+	    get_value(message, 4, strlen(FRG_TR), 6, INT_STRING, 256, 2346,
+		WLAN_HELP);
 	    FragmentationThreshold = atoi(message);
 	    m_fragment = 1;
 	    continue;
 	case 'R':
 	case 'r':
-//          make_field(3, 19, 6);
-	    get_value(message, 3, 19, 6);
+//          make_field(3, strlen(RTS_TR), 6);
+	    get_value(message, 3, strlen(RTS_TR), 6, INT_STRING, 0, 2347,
+		WLAN_HELP);
 	    RTSThreshold = atoi(message);
 	    m_rts = 1;
 	    continue;
+	case 'I':
+	case 'i':
+	    if (ap_type == ATMEL410)
+		continue;
+	    InterRoaming = on_off(strlen(INT_ROAMING), 15);
+	    clear_main_new(15, 16);
+	    print_menusel(15, 0, INT_ROAMING, (InterRoaming == 1) ? ON : OFF);
+	    m_inter_roaming = 1;
+	    continue;
+	case 'B':
+	case 'b':
+	    if (!IS_ATMEL12350_TELLUS)
+		continue;
+	    get_value(message, 16, strlen(BEACON_PER), 6, INT_STRING, 0, 65535,
+		WLAN_HELP);
+	    BeaconPeriod = atoi(message);
+	    m_beacon = 1;
+	    continue;
+	case 'D':
+	case 'd':
+	    if (!IS_ATMEL12350_TELLUS)
+		continue;
+	    get_value(message, 17, strlen(DTIM_I), 4, INT_STRING, 0, 255,
+		WLAN_HELP);
+	    dtim = atoi(message);
+	    m_dtim = 1;
+	    continue;
 	case 'w':
 	case 'W':
 	    i = 0;
@@ -327,7 +434,6 @@ void wireless()
 		varbinds[i].type = INT_VALUE;
 		i++;
 	    }
-
 	    if (m_basic_rates) {
 		for (m_basic_rates = 0; m_basic_rates < 2; m_basic_rates++) {
 		    c = basic_rates[3 - m_basic_rates];
@@ -402,23 +508,52 @@ void wireless()
 		i++;
 	    }
 	    if (m_essid) {
-		m_essid = strlen(domain);
+		c = strlen(domain);
 		varbinds[i].oid = operESSIDLength;
 		varbinds[i].len_oid = sizeof(operESSIDLength);
-		varbinds[i].value = &m_essid;
+		varbinds[i].value = (char *)&c;
 		varbinds[i].len_val = 1;
 		varbinds[i].type = INT_VALUE;
 		i++;
 		varbinds[i].oid = operESSID;
 		varbinds[i].len_oid = sizeof(operESSID);
 		varbinds[i].value = domain;
-		varbinds[i].len_val = m_essid;
+		varbinds[i].len_val = c;
 		varbinds[i].type = STRING_VALUE;
 		i++;
 	    }
+	    if (m_inter_roaming) {
+		varbinds[i].oid = operInterRoaming;
+		varbinds[i].len_oid = sizeof(operInterRoaming);
+		varbinds[i].value = (char *) &InterRoaming;
+		varbinds[i].len_val = 1;
+		varbinds[i].type = INT_VALUE;
+		i++;
+	    }
+	    if (m_beacon) {
+		int len_val;
+
+		len_val = (BeaconPeriod > 0x7fff) ? 3 :
+			    (BeaconPeriod > 0x7f) ? 2 : 1;
+		varbinds[i].oid = operBeaconPeriod;
+		varbinds[i].len_oid = sizeof(operBeaconPeriod);
+		varbinds[i].value = (char *) &BeaconPeriod;
+		varbinds[i].len_val = len_val;
+		varbinds[i].type = STRING_VALUE;
+		i++;
+	    }
+	    if (m_dtim) {
+		varbinds[i].oid = operDTIM;
+		varbinds[i].len_oid = sizeof(operDTIM);
+		varbinds[i].value = (char *) &dtim;
+		varbinds[i].len_val = 1;
+		varbinds[i].type = INT_VALUE;
+		i++;
+	    }
+
 	    print_help(WAIT_SET);
 	    if (snmp(varbinds, i, SET) <= 0) {
-		print_help(ERR_SET);
+		print_helperr(ERR_SET);
 		goto exit;
 	    }
 	    print_help(DONE_SET);
@@ -485,10 +620,12 @@ void nwn_wireless()
 
     unsigned short int i, RTSThreshold, FragmentationThreshold,
 	m_basic_rates = 0, authi;
-    char *auth[3] = { OSYS, SH_KEY, BOTH_TYPE }, *rates[3] = {
-    ON, OFF, BASIC}, message[1024], m_antenna[6] = {
-    0, 0, 0, 0, 0, 0}, antenna[6], channel_pref[2] = {
-    0, 0};
+    char *auth[3] = { OSYS, SH_KEY, BOTH_TYPE },
+	 *rates[3] = { ON, OFF, BASIC },
+	 message[1024],
+	 m_antenna[6] = { 0, 0, 0, 0, 0, 0 },
+	 antenna[6],
+	 channel_pref[2] = { 0, 0 };
     extern WINDOW *main_sub;
     char *domain, basic_rates[] =
 	{ 0, 0, 0, 0 }, *ap_name, ap_location[32], ap_contact[32],
@@ -524,7 +661,7 @@ void nwn_wireless()
     varbinds[7].len_oid = sizeof(operAccessPointName);
     print_help(WAIT_RET);
     if (snmp(varbinds, 8, GET) <= 0) {
-	print_help(ERR_RET);
+	print_helperr(ERR_RET);
 	goto exit;
     }
 
@@ -539,31 +676,31 @@ void nwn_wireless()
 	(*(varbinds[5].value) == 1) ? 0 : 1;
     memcpy(ap_name, varbinds[7].value, varbinds[7].len_val);
 
-    sprintf(message, CHANNEL, channel, 2407 + 5 * channel);
+    sprintf(message, "%s%02u (%u MHz)", CHANNEL, channel, 2407 + 5 * channel);
     mvwaddstr(main_sub, 0, 0, message);
     mvwaddstr(main_sub, 1, 0, ESSID);
-    for (i = 0; i < 32 && *(domain + i); i++) {
-	mvwaddch(main_sub, 1, i + 25, *(domain + i));
-    }
-    mvwaddstr(main_sub, 2, 0, BSS_NAME);
+    for (i = 0; i < 32 && domain[i]; i++)
+	mvwaddch(main_sub, 1, strlen(ESSID) + i, domain[i]);
+    mvwaddstr(main_sub, 2, 0, AP_NAME);
     for (i = 0; i < 32 && ap_name[i]; i++)
-	mvwaddch(main_sub, 2, i + 25, *(ap_name + i));
-    sprintf(message, RTS_TR, RTSThreshold);
+	mvwaddch(main_sub, 2, strlen(AP_NAME) + i, ap_name[i]);
+    sprintf(message, "%s%d", RTS_TR, RTSThreshold);
     mvwaddstr(main_sub, 3, 0, message);
-    sprintf(message, FRG_TR, FragmentationThreshold);
+    sprintf(message, "%s%d", FRG_TR, FragmentationThreshold);
     mvwaddstr(main_sub, 4, 0, message);
-    sprintf(message, AUTH, auth[authi]);
+    sprintf(message, "%s%s", AUTH, auth[authi]);
     mvwaddstr(main_sub, 6, 0, message);
     mvwaddstr(main_sub, 9, 0, RATES);
     mvwaddstr(main_sub, 10, 2, RATES_HEAD);
-    for(i=0; i<4; i++) {
-        sprintf(message, RATES_RECORD, i+1, (float) __rates[i]/2, basic(basic_rates[i]));
+    for(i = 0; i < 4; i++) {
+	sprintf(message, RATES_RECORD "%s", i + 1, (float) __rates[i] / 2,
+		basic(basic_rates[i]));
         mvwaddstr(main_sub, 11+i, 2, message);
     }
     
     reg_domain = get_RegDomain();
     if ((ch_num = ch_list(reg_domain, item_name)) == 0) {
-	print_help(ERR_RET);
+	print_helperr(ERR_RET);
 	goto exit;
     }
 
@@ -587,7 +724,7 @@ void nwn_wireless()
     varbinds[8].len_oid = sizeof(operAccessPointLocation);
 
     if (snmp(varbinds, 9, GET) <= 0) {
-	print_help(ERR_RET);
+	print_helperr(ERR_RET);
 	goto exit;
     }
 
@@ -603,25 +740,30 @@ void nwn_wireless()
     if (varbinds[8].len_val < 32)
 	ap_location[varbinds[8].len_val] = '\0';
 
-    sprintf(message, _("[L] AP Location:         %s"), ap_location);
+    sprintf(message, "%s%s", LOCATION, ap_location);
     mvwaddstr(main_sub, 7, 0, message);
-    sprintf(message, _("[E] AP Contact:          %s"), ap_contact);
+    sprintf(message, "%s%s", CONTACT, ap_contact);
     mvwaddstr(main_sub, 8, 0, message);
-    sprintf(message, TRAPS, (traps == 1) ? ON : OFF);
+    sprintf(message, "%s%s", TRAPS, (traps == 1) ? ON : OFF);
     mvwaddstr(main_sub, 5, 0, message);
 
     mvwaddstr(main_sub, 16, 0, _("Antenna Configuration:"));
-    sprintf(message, ANTENNA_RX,
-	    (antenna[2] == 1) ? ON : OFF, (antenna[3] == 1) ? ON : OFF);
+    sprintf(message, "%s %s %3s, %s %3s",
+	    ANTENNA_RX,
+	    ANTENNA_RX_LEFT, (antenna[2] == 1) ? ON : OFF,
+	    ANTENNA_RX_RIGHT, (antenna[3] == 1) ? ON : OFF);
     mvwaddstr(main_sub, 17, 1, message);
-    sprintf(message, ANTENNA_TX,
-	    (antenna[0] == 1) ? ON : OFF, (antenna[1] == 1) ? ON : OFF);
+    sprintf(message, "%s %s %3s, %s %3s",
+	    ANTENNA_TX,
+	    ANTENNA_TX_LEFT, (antenna[0] == 1) ? ON : OFF,
+	    ANTENNA_TX_RIGHT, (antenna[1] == 1) ? ON : OFF);
     mvwaddstr(main_sub, 18, 1, message);
-    sprintf(message, ANTENNA_DV,
-	    (antenna[4] == 1) ? ON : OFF, (antenna[5] == 1) ? ON : OFF);
+    sprintf(message, "%s %s %3s, %s %3s",
+	    ANTENNA_DV,
+	    ANTENNA_DV_LEFT, (antenna[4] == 1) ? ON : OFF,
+	    ANTENNA_DV_RIGHT, (antenna[5] == 1) ? ON : OFF);
     mvwaddstr(main_sub, 19, 1, message);
 
-
     print_title(_("General Options"));
     print_help(_
 	       ("UIOPTY - antenna; SCANLEDFR1234 - options; W - write conf; Q - quit to menu"));
@@ -634,77 +776,66 @@ void nwn_wireless()
 	switch (c) {
 	case 'i':
 	case 'I':
-	    antenna[3] = on_off(43, 17);
-	    sprintf(message,
-		    ANTENNA_RX,
-		    (antenna[2] == 1) ? ON : OFF,
-		    (antenna[3] == 1) ? ON : OFF);
-	    mvwaddstr(main_sub, 17, 1, message);
+	    i = strlen(ANTENNA_RX) + 1 +
+		strlen(ANTENNA_RX_LEFT) + 6 +
+		strlen(ANTENNA_RX_RIGHT) + 2;
+	    antenna[3] = on_off(i, 17);
+	    sprintf(message, "%3s", (antenna[3] == 1) ? ON : OFF);
+	    print_menusel(17, i, NULL, message);
 	    m_antenna[3] = 1;
-	    wrefresh(main_sub);
 	    continue;
 	case 'u':
 	case 'U':
-	    antenna[2] = on_off(28, 17);
+	    i = strlen(ANTENNA_RX) + 1 +
+		strlen(ANTENNA_RX_LEFT) + 2;
+	    antenna[2] = on_off(i, 17);
+	    sprintf(message, "%3s", (antenna[2] == 1) ? ON : OFF);
+	    print_menusel(17, i, NULL, message);
 	    m_antenna[2] = 1;
-	    sprintf(message,
-		    ANTENNA_RX,
-		    (antenna[2] == 1) ? ON : OFF,
-		    (antenna[3] == 1) ? ON : OFF);
-	    mvwaddstr(main_sub, 17, 1, message);
-	    wrefresh(main_sub);
 	    continue;
 	case 'p':
 	case 'P':
-	    antenna[1] = on_off(43, 18);
+	    i = strlen(ANTENNA_TX) + 1 +
+		strlen(ANTENNA_TX_LEFT) + 6 +
+		strlen(ANTENNA_TX_RIGHT) + 2;
+	    antenna[1] = on_off(i, 18);
+	    sprintf(message, "%3s", (antenna[1] == 1) ? ON : OFF);
+	    print_menusel(18, i, NULL, message);
 	    m_antenna[1] = 1;
-	    sprintf(message,
-		    ANTENNA_TX,
-		    (antenna[0] == 1) ? ON : OFF,
-		    (antenna[1] == 1) ? ON : OFF);
-	    mvwaddstr(main_sub, 18, 1, message);
-	    wrefresh(main_sub);
 	    continue;
 	case 'o':
 	case 'O':
-	    antenna[0] = on_off(28, 18);
+	    i = strlen(ANTENNA_TX) + 1 +
+		strlen(ANTENNA_TX_LEFT) + 2;
+	    antenna[0] = on_off(i, 18);
+	    sprintf(message, "%3s", (antenna[0] == 1) ? ON : OFF);
+	    print_menusel(18, i, NULL, message);
 	    m_antenna[0] = 1;
-	    sprintf(message,
-		    ANTENNA_TX,
-		    (antenna[0] == 1) ? ON : OFF,
-		    (antenna[1] == 1) ? ON : OFF);
-	    mvwaddstr(main_sub, 18, 1, message);
-	    wrefresh(main_sub);
 	    continue;
 	case 'T':
 	case 't':
-	    antenna[4] = on_off(28, 19);
+	    i = strlen(ANTENNA_DV) + 1 +
+		strlen(ANTENNA_DV_LEFT) + 2;
+	    antenna[4] = on_off(i, 19);
+	    sprintf(message, "%3s", (antenna[4] == 1) ? ON : OFF);
+	    print_menusel(19, i, NULL, message);
 	    m_antenna[4] = 1;
-	    sprintf(message,
-		    ANTENNA_DV,
-		    (antenna[4] == 1) ? ON : OFF,
-		    (antenna[5] == 1) ? ON : OFF);
-	    mvwaddstr(main_sub, 19, 1, message);
-	    wrefresh(main_sub);
 	    continue;
 	case 'Y':
 	case 'y':
-	    antenna[5] = on_off(43, 19);
+	    i = strlen(ANTENNA_DV) + 1 +
+		strlen(ANTENNA_DV_LEFT) + 6 +
+		strlen(ANTENNA_DV_RIGHT) + 2;
+	    antenna[5] = on_off(i, 19);
+	    sprintf(message, "%3s", (antenna[5] == 1) ? ON : OFF);
+	    print_menusel(19, i, NULL, message);
 	    m_antenna[5] = 1;
-	    sprintf(message,
-		    ANTENNA_DV,
-		    (antenna[4] == 1) ? ON : OFF,
-		    (antenna[5] == 1) ? ON : OFF);
-	    mvwaddstr(main_sub, 19, 1, message);
-	    wrefresh(main_sub);
 	    continue;
 	case 'S':
 	case 's':
-	    traps = on_off(21, 5);
+	    traps = on_off(strlen(TRAPS), 5);
 	    clear_main_new(5, 6);
-	    sprintf(message, TRAPS, (traps == 1) ? ON : OFF);
-	    mvwaddstr(main_sub, 5, 0, message);
-	    wrefresh(main_sub);
+	    print_menusel(5, 0, TRAPS, (traps == 1) ? ON : OFF);
 	    m_traps = 1;
 	    continue;
 	case 'Q':
@@ -725,64 +856,60 @@ void nwn_wireless()
                 case 2:
                     basic_rates[i-1] = __rates[i-1] + 0x80;
                 }
-            sprintf(message, RATES_RECORD, i, (float) __rates[i-1]/2, basic(basic_rates[i-1]));
-            clear_main_new(10+i, 11+i);
-            mvwaddstr(main_sub, 10+i, 2, message);
+            clear_main_new(10 + i, 11 + i);
+            sprintf(message, RATES_RECORD, i, (float) __rates[i-1] / 2);
+	    print_menusel(10 + i, 2, message, basic(basic_rates[i-1]));
             m_basic_rates = 1;
-	    wrefresh(main_sub);
             break;
 	case 'C':
 	case 'c':
-	    channel = menu_choose(21, 0, item_name, ch_num) + first_channel;
+	    channel = menu_choose(strlen(CHANNEL), 0, item_name, ch_num)
+		+ first_channel;
+	    sprintf(message, "%02u (%u MHz)", channel, 2407 + 5 * channel);
+	    print_menusel(0, 0, CHANNEL, message);
 	    m_channel = 1;
-	    sprintf(message, CHANNEL, channel, 2407 + 5 * channel);
-	    mvwaddstr(main_sub, 0, 0, message);
-	    wrefresh(main_sub);
 	    continue;
 	case 'A':
 	case 'a':
-	    authi = menu_choose(14, 6, auth, 3);
-	    m_auth = 1;
+	    authi = menu_choose(strlen(AUTH), 6, auth, 3);
 	    clear_main_new(6, 7);
-	    sprintf(message, AUTH, auth[authi]);
-	    mvwaddstr(main_sub, 6, 0, message);
-	    wrefresh(main_sub);
+	    print_menusel(6, 0, AUTH, auth[authi]);
+	    m_auth = 1;
 	    continue;
 	case 'N':
 	case 'n':
-	    get_value(ap_name, 2, 25, 32);
+	    get_value(ap_name, 2, strlen(AP_NAME), 32, ANY_STRING, 0, 0, NULL);
 	    m_ap_name = 1;
 	    continue;
 	case 'L':
 	case 'l':
-	    get_value(ap_location, 7, 25, 32);
+	    get_value(ap_location, 7, strlen(LOCATION), 32, ANY_STRING, 0, 0,
+		NULL);
 	    m_ap_location = 1;
 	    continue;
 	case 'E':
 	case 'e':
-	    get_value(ap_contact, 8, 25, 32);
+	    get_value(ap_contact, 8, strlen(CONTACT), 32, ANY_STRING, 0, 0,
+		NULL);
 	    m_ap_contact = 1;
 	    continue;
 	case 'D':
 	case 'd':
-	    get_value(domain, 1, 25, 32);
+	    get_value(domain, 1, strlen(ESSID), 32, ANY_STRING, 0, 0, NULL);
 	    m_essid = 1;
 	    continue;
 	case 'F':
 	case 'f':
-	    get_value(message, 4, 29, 6);
+	    get_value(message, 4, strlen(FRG_TR), 6, INT_STRING, 256, 2346,
+		WLAN_HELP);
 	    FragmentationThreshold = atoi(message);
-	    if (FragmentationThreshold < 256
-		|| FragmentationThreshold > 2346)
-		FragmentationThreshold = 2346;
 	    m_fragment = 1;
 	    continue;
 	case 'R':
 	case 'r':
-	    get_value(message, 3, 19, 6);
+	    get_value(message, 3, strlen(RTS_TR), 6, INT_STRING, 0, 2347,
+		WLAN_HELP);
 	    RTSThreshold = atoi(message);
-	    if (RTSThreshold > 2347)
-		RTSThreshold = 2347;
 	    m_rts = 1;
 	    continue;
 	case 'w':
@@ -815,7 +942,7 @@ void nwn_wireless()
 
 		print_help(WAIT_SET);
 		if (snmp(varbinds, i, SET) <= 0) {
-		    print_help(ERR_SET);
+		    print_helperr(ERR_SET);
 		    goto exit;
 		}
 	    i = 0;
@@ -838,7 +965,7 @@ void nwn_wireless()
 	    }
 		print_help(WAIT_SET);
 		if (snmp(varbinds, i, SET) <= 0) {
-		    print_help(ERR_SET);
+		    print_helperr(ERR_SET);
 		    goto exit;
 		}
 	    i = 0;
@@ -865,7 +992,7 @@ void nwn_wireless()
 
 		print_help(WAIT_SET);
 		if (snmp(varbinds, i, SET) <= 0) {
-		    print_help(ERR_SET);
+		    print_helperr(ERR_SET);
 		    goto exit;
 		}
 
@@ -907,7 +1034,7 @@ void nwn_wireless()
 	    }
 		print_help(WAIT_SET);
 		if (snmp(varbinds, i, SET) <= 0) {
-		    print_help(ERR_SET);
+		    print_helperr(ERR_SET);
 		    goto exit;
 		}
 	    c = 0;
@@ -923,7 +1050,7 @@ void nwn_wireless()
 		}
 		print_help(WAIT_SET);
 		if (snmp(varbinds, c, SET) <= 0) {
-		    print_help(ERR_SET);
+		    print_helperr(ERR_SET);
 		    goto exit;
 		}
 	    c = 0;
@@ -939,7 +1066,7 @@ void nwn_wireless()
 		}
 		print_help(WAIT_SET);
 		if (snmp(varbinds, c, SET) <= 0) {
-		    print_help(ERR_SET);
+		    print_helperr(ERR_SET);
 		    goto exit;
 	    }
 	    i = 0;
@@ -954,7 +1081,7 @@ void nwn_wireless()
 	    }
 		print_help(WAIT_SET);
 		if (snmp(varbinds, i, SET) <= 0) {
-		    print_help(ERR_SET);
+		    print_helperr(ERR_SET);
 		    goto exit;
 		}
 
